@@ -18,6 +18,7 @@ import logging
 import pytz
 
 from django.conf import settings
+from django.utils.module_loading import import_string  # noqa
 from django.utils.translation import ugettext_lazy as _
 from keystoneclient import exceptions as keystone_exceptions
 
@@ -47,7 +48,7 @@ class KeystoneBackend(object):
                 ['openstack_auth.plugin.password.PasswordPlugin',
                  'openstack_auth.plugin.token.TokenPlugin'])
 
-            self._auth_plugins = [utils.import_string(p)() for p in plugins]
+            self._auth_plugins = [import_string(p)() for p in plugins]
 
         return self._auth_plugins
 
@@ -128,15 +129,47 @@ class KeystoneBackend(object):
         # Check expiry for our unscoped auth ref.
         self.check_auth_expiry(unscoped_auth_ref)
 
+        # domain support can require domain scoped tokens to perform
+        # identity operations depending on the policy files being used
+        # for keystone.
+        domain_auth = None
+        domain_auth_ref = None
+        if utils.get_keystone_version() >= 3 and 'user_domain_name' in kwargs:
+            try:
+                token = unscoped_auth_ref.auth_token
+                domain_auth = utils.get_token_auth_plugin(
+                    auth_url,
+                    token,
+                    domain_name=kwargs['user_domain_name'])
+                domain_auth_ref = domain_auth.get_access(session)
+            except Exception:
+                LOG.debug('Error getting domain scoped token.', exc_info=True)
+
         projects = plugin.list_projects(session,
                                         unscoped_auth,
                                         unscoped_auth_ref)
         # Attempt to scope only to enabled projects
         projects = [project for project in projects if project.enabled]
 
-        # Abort if there are no projects for this user
-        if not projects:
+        # Abort if there are no projects for this user and a valid domain
+        # token has not been obtained
+        #
+        # The valid use cases for a user login are:
+        #    Keystone v2: user must have a role on a project and be able
+        #                 to obtain a project scoped token
+        #    Keystone v3: 1) user can obtain a domain scoped token (user
+        #                    has a role on the domain they authenticated to),
+        #                    only, no roles on a project
+        #                 2) user can obtain a domain scoped token and has
+        #                    a role on a project in the domain they
+        #                    authenticated to (and can obtain a project scoped
+        #                    token)
+        #                 3) user cannot obtain a domain scoped token, but can
+        #                    obtain a project scoped token
+        if not projects and not domain_auth_ref:
             msg = _('You are not authorized for any projects.')
+            if utils.get_keystone_version() >= 3:
+                msg = _('You are not authorized for any projects or domains.')
             raise exceptions.KeystoneAuthException(msg)
 
         # the recent project id a user might have set in a cookie
@@ -171,8 +204,15 @@ class KeystoneBackend(object):
             else:
                 break
         else:
-            msg = _("Unable to authenticate to any available projects.")
-            raise exceptions.KeystoneAuthException(msg)
+            # if the user can't obtain a project scoped token, set the scoped
+            # token to be the domain token, if valid
+            if domain_auth_ref:
+                scoped_auth = domain_auth
+                scoped_auth_ref = domain_auth_ref
+            else:
+                # if no domain or project token for user, abort
+                msg = _("Unable to authenticate to any available projects.")
+                raise exceptions.KeystoneAuthException(msg)
 
         # Check expiry for our new scoped token.
         self.check_auth_expiry(scoped_auth_ref)
@@ -188,6 +228,19 @@ class KeystoneBackend(object):
 
         if request is not None:
             request.session['unscoped_token'] = unscoped_token
+            if domain_auth_ref:
+                # check django session engine, if using cookies, this will not
+                # work, as it will overflow the cookie so don't add domain
+                # scoped token to the session and put error in the log
+                if utils.using_cookie_backed_sessions():
+                    LOG.error('Using signed cookies as SESSION_ENGINE with '
+                              'OPENSTACK_KEYSTONE_MULTIDOMAIN_SUPPORT is '
+                              'enabled. This disables the ability to '
+                              'perform identity operations due to cookie size '
+                              'constraints.')
+                else:
+                    request.session['domain_token'] = domain_auth_ref
+
             request.user = user
             timeout = getattr(settings, "SESSION_TIMEOUT", 3600)
             token_life = user.token.expires - datetime.datetime.now(pytz.utc)
