@@ -30,6 +30,7 @@ from django.views.decorators.csrf import csrf_protect  # noqa
 from django.views.decorators.debug import sensitive_post_parameters  # noqa
 from keystoneclient.auth import token_endpoint
 from keystoneclient import exceptions as keystone_exceptions
+import six
 
 from openstack_auth import exceptions
 from openstack_auth import forms
@@ -58,12 +59,10 @@ def login(request, template_name=None, extra_context=None, **kwargs):
     # If the user enabled websso and selects default protocol
     # from the dropdown, We need to redirect user to the websso url
     if request.method == 'POST':
-        protocol = request.POST.get('auth_type', 'credentials')
-        if utils.is_websso_enabled() and protocol != 'credentials':
-            region = request.POST.get('region')
-            origin = utils.build_absolute_uri(request, '/auth/websso/')
-            url = ('%s/auth/OS-FEDERATION/websso/%s?origin=%s' %
-                   (region, protocol, origin))
+        auth_type = request.POST.get('auth_type', 'credentials')
+        if utils.is_websso_enabled() and auth_type != 'credentials':
+            auth_url = request.POST.get('region')
+            url = utils.get_websso_url(request, auth_url, auth_type)
             return shortcuts.redirect(url)
 
     if not request.is_ajax():
@@ -140,7 +139,7 @@ def websso(request):
         request.user = auth.authenticate(request=request, auth_url=auth_url,
                                          token=token)
     except exceptions.KeystoneAuthException as exc:
-        msg = 'Login failed: %s' % unicode(exc)
+        msg = 'Login failed: %s' % six.text_type(exc)
         res = django_http.HttpResponseRedirect(settings.LOGIN_URL)
         res.set_cookie('logout_reason', msg, max_age=10)
         return res
@@ -166,9 +165,17 @@ def logout(request, login_url=None, **kwargs):
         {'username': request.user.username}
     LOG.info(msg)
     endpoint = request.session.get('region_endpoint')
+
+    # delete the project scoped token
     token = request.session.get('token')
     if token and endpoint:
         delete_token(endpoint=endpoint, token_id=token.id)
+
+    # delete the domain scoped token if set
+    domain_token = request.session.get('domain_token')
+    if domain_token and endpoint:
+        delete_token(endpoint=endpoint, token_id=domain_token.auth_token)
+
     """ Securely logs a user out. """
     return django_auth_views.logout_then_login(request, login_url=login_url,
                                                **kwargs)
@@ -176,8 +183,6 @@ def logout(request, login_url=None, **kwargs):
 
 def delete_token(endpoint, token_id):
     """Delete a token."""
-    utils.remove_project_cache(token_id)
-
     try:
         endpoint = utils.fix_auth_url_version(endpoint)
 
